/*
    Copyright 2008-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef VALUELOGGER_H
#define VALUELOGGER_H

#include <QtCore/QObject>
#include <QtCore/QMap>
#include <QtGui/QWidget>
#include <QtGui/QGraphicsView>
#include <QtGui/QGraphicsScene>

class AbstractLogging;

class Logger : public QObject
{
	Q_OBJECT
	friend class AbstractLogging;

	protected:
		Logger( AbstractLogging* );
		double getValue();

	public slots:
		void logValue( double n );

	private:
		double _value;
		int _counter;
};

class AbstractLogging
{
	public:
		AbstractLogging();

		Logger* getLogger( QString );

		void addLogger( QString, QObject*, const char* signal );

	protected:
		typedef QMap<QString,double> logvalues_t;
		logvalues_t getLogValues();

	private:
		QMap<QString,Logger*> _loggers;
};

class ValueLogging : public QObject, public AbstractLogging
{
	Q_OBJECT
	public:
		ValueLogging( QObject* );

	private slots:
		void loggingTimeout();

};

class GuiLoggingView;

class GuiLogging : public QWidget, public AbstractLogging
{
	Q_OBJECT
	public:
		GuiLogging( QWidget* );
		~GuiLogging();

	private slots:
		void loggingTimeout();

	private:
		QMap<QString,GuiLoggingView*> _views;
};

class GuiLoggingView : public QGraphicsView
{
	Q_OBJECT
	public:
		GuiLoggingView( GuiLogging* );

	public slots:
		void addValue( double );

	private slots:
		void autoZoom( const QRectF& );

	private:
		QGraphicsScene* _scene;
		QGraphicsItemGroup* _group;
};

#endif//VALUELOGGER_H
