/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "abstracttreeitem.h"

namespace NSCore {

	AbstractTreeItem::AbstractTreeItem( QString name, AbstractTreeItem* parent )
		: _parent( parent )
		, _name( name )
	{
		//qDebug() << "AbstractTreeItem::AbstractTreeItem(" << name << "," << parent << ")";
		if ( _parent )
			_parent->_childs[ _name ] = this;
	}
	AbstractTreeItem::AbstractTreeItem( AbstractTreeItem& item, AbstractTreeItem* parent )
		: _parent( parent )
		, _name( item._name )
	{
		qDebug() << "AbstractTreeItem::AbstractTreeItem(" << &item << "," << parent << ")\n ! Copy constructor is not working right!";
		if ( _parent )
			_parent->_childs[ _name ] = this;
	}
	AbstractTreeItem::~AbstractTreeItem() {
		//qDebug() << "AbstractTreeItem::~AbstractTreeItem() _name=" << _name;
		while ( _childs.size() > 0 ) {
			delete ( *_childs.begin() ).second;
			_childs.erase( _childs.begin() );
		}
		//qDebug() << "1";
		//if ( _parent )
		//	_parent->_childs.erase( _parent->_childs.find( _name ) );
		//qDebug() << "AbstractTreeItem::~AbstractTreeItem() finished";
	}
	QString AbstractTreeItem::path() const {
		QString ret="";
		AbstractTreeItem* item = dynamic_cast<AbstractTreeItem*>( _parent );
		if ( item )
			ret = item->path() + "/";
		return ret + name();
	}
	AbstractTreeItem* AbstractTreeItem::getNamedChild( QString n ) const {
		//qDebug() << "AbstractTreeItem::getNamedChild(" << n << ")";
		//qDebug() << " own name=" << _name << " childlist=" << childlist().join( "," );
		if ( n.startsWith( "/" ) ) n.remove( 0, 1 );
		if ( n.startsWith( _name ) )
			n.remove( 0, _name.size() );
		if ( n.startsWith( "/" ) ) n.remove( 0, 1 );
		//qDebug() << " n =" << n;
		//qDebug() << " n.indexOf(\"/\" ) =" << n.indexOf( "/" );
		//qDebug() << " childname =" << n.left( n.indexOf( "/" ) );
		AbstractTreeItem* item = child( n.left( n.indexOf( "/" ) ) );
		if ( item != 0 ) {
			n.remove( 0, item->name().size() +1 );
			//qDebug() << " n =" << n;
			if ( n == "" )
				return item;
			else
				return item->getNamedChild( n );
		}
		//qDebug() << "AbstractTreeItem::getNamedChild(" << n << ")  returning 0!";
		//qDebug() << " own name=" << _name << " childlist=" << childlist().join( "," );
		return 0;
	}
	AbstractTreeItem* AbstractTreeItem::child( QString n ) const {
		childlist_t::const_iterator it = _childs.find( n );
		if ( it == _childs.end() )
			return 0;
		return ( *it ).second;
	}

};

