/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef GUISERVER_INTERFACE_H
#define GUISERVER_INTERFACE_H

#include <QtCore/QString>
#include <QtCore/QStringList>

namespace NSCore {

	/**
	 * @brief Abstract interface for a GUI server
	 *
	 */
	class GuiServer_Interface
	{
		public:
			virtual ~GuiServer_Interface() {}

			virtual void message( const QString& title, const QString& text ) const =0;
			virtual bool messageYesNo( const QString& title, const QString& text ) const =0;
			virtual bool messageOkCancel( const QString& title, const QString& text ) const =0;

			virtual double getDouble( const QString& title, const QString& label, double initvalue, double minvalue, double maxvalue ) const =0;
			virtual int getInt( const QString& title, const QString& label, int initvalue, int minvalue, int maxvalue ) const =0;
			virtual QString getString( const QString& title, const QString& label, const QString& initvalue ) const =0;
			virtual QString getItem( const QString& title, const QString& label, const QStringList& list ) const =0;

			struct ListItem {
				QString title;
				QString help;
				QString icon;
				QStringList categories;
			};
			virtual QString getItem( const QString& title, const QString& desc, const QList<ListItem> items ) const =0;

			enum FileType {
				Read,
				Write,
				Dir
			};

			virtual QString getFileName( const QString& title, const QString& dir, const QString& filter, const FileType& ) =0;

	};

	bool operator==( const GuiServer_Interface::ListItem& first, const GuiServer_Interface::ListItem& second );
};

#endif // GUISERVER_INTERFACE_H

