/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "diskio_gui.h"
#include "diskio_gui.moc"

#include "abstractdiskio.h"

#include <QtGui/QPushButton>
#include <QtGui/QGridLayout>
#include <QtGui/QFileDialog>
#include <QtGui/QSpinBox>
#include <QtGui/QLabel>
#include <QtCore/QFile>
#include <QtGui/QProgressBar>
#include <QtGui/QAction>
#include <QtGui/QToolBar>
#include <QtCore/QDebug>
#include <QtGui/QMenu>
#include <QtGui/QContextMenuEvent>
#include <QtGui/QIcon>
#include <QtGui/QLineEdit>
#include <QtCore/QDateTime>
#include <akhline.h>

namespace NSDiskIO {

	DiskReaderGui::DiskReaderGui( AbstractReader* reader, QWidget* p )
		: QFrame( p )
		, _reader( reader )
		, _toolbar( new QToolBar( "DiskReader" ) )
	{
		QGridLayout* layout = new QGridLayout( this );
		layout->setMargin( 5 );
		layout->setSpacing( 5 );

		QPushButton* file = new QPushButton( QIcon( ":/icons/kde/cr22-action-fileopen.png" ), "Open File...", this );
		connect( file, SIGNAL( clicked() ), this, SLOT( openfile() ) );
		QPushButton* reset = new QPushButton( QIcon( ":/icons/kde/cr22-action-player_start.png" ), "Reset file", this );
		connect( reset, SIGNAL( clicked() ), _reader, SLOT( reset() ) );
		QPushButton* readn = new QPushButton( QIcon( ":/icons/kde/cr22-action-player_fwd.png" ), "Read!", this );
		connect( readn, SIGNAL( clicked() ), this, SLOT( readn() ) );
		QPushButton* readAll = new QPushButton( QIcon( ":/icons/kde/cr22-action-player_end.png" ), "Read ALL!", this );
		connect( readAll, SIGNAL( clicked() ), _reader, SLOT( readAllDataSets() ) );
		_spinbox = new QSpinBox( this );
		_spinbox->setMinimum( 1 );
		_spinbox->setMaximum( int( 1E8 ) );
		_spinbox->setSingleStep( 10 );
		QLabel* lbl = new QLabel( "Datasets to read:", this );
		QProgressBar* bar = new QProgressBar( this );
		connect( _reader, SIGNAL( percent( int ) ), bar, SLOT( setValue( int ) ) );

		layout->addWidget( file, 0,0 );
		layout->addWidget( reset, 0,1 );
		layout->addWidget( readAll, 0,2 );
		layout->addWidget( lbl, 1,0 );
		layout->addWidget( _spinbox, 1,1 );
		layout->addWidget( readn, 1,2 );
		layout->addWidget( bar, 2,0, 1,3 );

		_fileopen = new QAction( QIcon( ":/icons/kde/cr22-action-fileopen.png" ), "Open File...", this );
		connect( _fileopen, SIGNAL( triggered() ), this, SLOT( openfile() ) );

		_readAll = new QAction( QIcon( ":/icons/kde/cr22-action-player_end.png" ), "Read ALL", this );
		connect( _readAll, SIGNAL( triggered() ), _reader, SLOT( readAllDataSets() ) );

		_readN = new QAction( QIcon( ":/icons/kde/cr22-action-player_fwd.png" ), "Read!", this );
		connect( _readN, SIGNAL( triggered() ), this, SLOT( readn() ) );

		_reset = new QAction( QIcon( ":/icons/kde/cr22-action-player_start.png" ), "Reset file to beginning", this );
		connect( _reset, SIGNAL( triggered() ), _reader, SLOT( reset() ) );

		_stopreading = new QAction( QIcon( ":/icons/kde/cr22-action-player_stop.png" ), "Stop reading", this );
		connect( _stopreading, SIGNAL( triggered() ), this /*_reader*/, SLOT( stopReading() ) );
		//_stopreading->setEnabled( false );

		{
			_toolbar->addWidget( new QLabel( "DiskReader:", _toolbar ) );
			_toolbar->addAction( _fileopen );
			_toolbar->addSeparator();
			_toolbar->addAction( _reset );
			_toolbar->addAction( _readN );
			_toolbar->addAction( _readAll );
			_toolbar->addAction( _stopreading );
			_toolbar->addSeparator();
			QProgressBar* bar = new QProgressBar( _toolbar );
			connect( _reader, SIGNAL( percent( int ) ), bar, SLOT( setValue( int ) ) );
			_toolbar->addWidget( bar );
			/*QLabel* time = new QLabel( "<elapsed time>", _toolbar );
			time->setFrameStyle( QFrame::Sunken|QFrame::Panel );
			time->setLineWidth( 1 );
			time->setMargin( 2 );
			connect( _reader, SIGNAL( elapsedTime( QString ) ), time, SLOT( setText( const QString& ) ) );
			_toolbar->addWidget( time );*/

			this->addAction( _fileopen );
			this->addAction( _readN );
			this->addAction( _readAll );
			this->addAction( _reset );
			this->addAction( _stopreading );
		}

		//connect( _reader, SIGNAL( startedRead() ), this, SLOT( startedReading() ) );
		//connect( _reader, SIGNAL( stoppedRead() ), this, SLOT( stoppedReading() ) );
		connect( this, SIGNAL( readn( int ) ), _reader, SLOT( readNDataSets( int ) ) );
	}

	void DiskReaderGui::readn() {
		emit readn( _spinbox->value() );
	}
	void DiskReaderGui::openfile() {
		QString filename = QFileDialog::getOpenFileName( this, "Open file...", _reader->filename(), _reader->filter() + "\nAll Files (*.*)" );
		if ( ! filename.isEmpty() && QFile::exists( filename ) )
			_reader->setFile( filename );
	}
	void DiskReaderGui::contextMenuEvent( QContextMenuEvent* ev ) {
		//qDebug() << "DiskReaderGui::contextMenuEvent(" << ev << ")";
		ev->accept();
		QMenu::exec( this->actions(), ev->globalPos() );
	}
	void DiskReaderGui::startedReading() {
		/*_fileopen->setEnabled( false );
		_readAll->setEnabled( false );
		_readN->setEnabled( false );
		_reset->setEnabled( false );
		_stopreading->setEnabled( true );*/
	}
	void DiskReaderGui::stoppedReading() {
		/*_fileopen->setEnabled( true );
		_readAll->setEnabled( true );
		_readN->setEnabled( true );
		_reset->setEnabled( true );
		_stopreading->setEnabled( false );*/
	}
	void DiskReaderGui::stopReading() {
		_reader->stopReading();
	}


	DiskWriterGui::DiskWriterGui( AbstractWriter* writer, QWidget* p ) : QFrame( p ), _writer( writer ) {
		QGridLayout* layout = new QGridLayout( this );
		layout->setSpacing( 5 );
		layout->setMargin( 5 );

		_lblcount = new QLabel( "Written 0 datasets so far.", this );
		connect( writer, SIGNAL( wroteDataSet( int ) ), this, SLOT( wroteDataSets( int ) ) );
		QPushButton* btn1 = new QPushButton( "Reset Counter", this );
		connect( btn1, SIGNAL( clicked() ), _writer, SLOT( resetCounter() ) );

		_inputprefix = new QLineEdit( "epos", this );
		QPushButton* btn2 = new QPushButton( "Set as new Filename", this );
		connect( btn2, SIGNAL( clicked() ), this, SLOT( setNewFilename() ) );
		QPushButton* btn3 = new QPushButton( "Close File", this );
		connect( btn3, SIGNAL( clicked() ), this, SLOT( closeFile() ) );

		QLabel *lbl1 = new QLabel( "Number of datasets to write:", this );
		_spinboxnumberofdatasets = new QSpinBox( this );
		_spinboxnumberofdatasets->setMinimum( 1 );
		_spinboxnumberofdatasets->setMaximum( int( 1E8 ) );
		_spinboxnumberofdatasets->setSingleStep( 100 );

		layout->setRowStretch( 0, 20 );
		layout->addWidget( _lblcount, 1,0 );
		layout->addWidget( btn1, 2,0 );
		layout->addWidget( new AKHLine( this ), 3,0 );
		layout->addWidget( _inputprefix, 4,0 );
		layout->addWidget( btn2, 5,0 );
		layout->addWidget( btn3, 6, 0 );
		layout->addWidget( new AKHLine( this ), 7,0 );
		layout->addWidget( lbl1, 8,0 );
		layout->addWidget( _spinboxnumberofdatasets, 9,0 );
		layout->setRowStretch( 10, 20 );
	}

	void DiskWriterGui::wroteDataSets( int n ) {
		_lblcount->setText( QString( "Written %1 datasets so far." ).arg( n ) );
		if ( n > _spinboxnumberofdatasets->value() )
			emit numberOfDataSetsReached();
	}

	void DiskWriterGui::setNewFilename() {
		_writer->setFile( _inputprefix->text() + QDateTime::currentDateTime().toString( "-yyyy-MM-dd-hh:mm:ss" ) );
	}

	void DiskWriterGui::closeFile() {
		_writer->setFile( "" );
	}
};

