/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "fancylistview.h"
#include "fancylistview.moc"
#include "fancylistdialog.h"

#include <QtCore/QDebug>
#include <QtCore/QVariant>

Q_DECLARE_METATYPE( NSCore::GuiServer_Interface::ListItem )

#include <QtGui/QWidget>

#include <QtGui/QMessageBox>
#include <QtGui/QInputDialog>
#include <QtGui/QHeaderView>
#include <QtGui/QTreeWidgetItem>
#include <QtGui/QPushButton>
#include <QtGui/QPainter>

#include <QtGui/QTextDocument>
#include <QtGui/QAbstractTextDocumentLayout>

#include "collapsiblewidget.h"


FancyListDialog::FancyListDialog( const QString& title, const QString& desc, QWidget* p ) : QDialog( p )
{
	d = new Ui_FancyListDialog;
	d->setupUi( this );
	d->label->setText( desc );
	setWindowTitle( title );

	d->buttonBox->button( QDialogButtonBox::Ok )->setEnabled( false );

	_model = new QStandardItemModel( 0, 1, this );
	d->treeView->setModel( _model );
	d->treeView->header()->hide();
	connect( d->treeView->selectionModel(), SIGNAL( currentChanged( const QModelIndex&, const QModelIndex& ) ), this, SLOT( currentChanged( const QModelIndex&, const QModelIndex& ) ) );

#ifdef OWNVIEW
	d->frame->setModel( _model );
#else
	d->frame->hide();
#endif

	d->search->setFocus( Qt::OtherFocusReason );
}
FancyListDialog::~FancyListDialog() {
	delete d;
}
void FancyListDialog::currentChanged( const QModelIndex& _current, const QModelIndex& _previous ) {
	//qDebug() << "FancyListDialog::currentChanged(" << _current << "," << _previous << ")";
	QStandardItem* current = _model->itemFromIndex( _current ); //static_cast<QStandardItem*>( _current.internalPointer() );
	QStandardItem* previous = _model->itemFromIndex( _previous ); //static_cast<QStandardItem*>( _previous.internalPointer() );
	//qDebug() << " current=" << current << " previous=" << previous;

	d->buttonBox->button( QDialogButtonBox::Ok )->setEnabled( _items.contains( current ) );
	if ( _categories.contains( current ) ) {
		d->treeView->expand( _current );
		if ( _categories.contains( previous ) )
			d->treeView->collapse( _previous );
		else
			if ( previous != 0 && _categories.contains( previous->parent() ) )
				d->treeView->collapse( _previous.parent() );
	}
	if ( _items.contains( current ) ) {
		currentItem = current->text();
	} else
		currentItem = "";
}
void FancyListDialog::on_search_textEdited( const QString& search ) {
	//qDebug() << "FancyListDialog::on_search_textEdited(" << search << ")";
	QList<QStandardItem*> found = _model->findItems( search, Qt::MatchContains | Qt::MatchWrap | Qt::MatchRecursive );
	if ( found.size() > 0 )
		d->treeView->setCurrentIndex( _model->indexFromItem( found.first() ) );
}
void FancyListDialog::addItems( QList<NSCore::GuiServer_Interface::ListItem> items ) {
	//qDebug() << "FancyListDialog::addItems( <List of" << items.size() << "items> )";
	foreach( NSCore::GuiServer_Interface::ListItem item, items ) {
		if ( item.categories.size() == 0 )
			item.categories.append( "No category" );
		foreach( QString cat, item.categories ) {
			QStandardItem* treeitem = new QStandardItem( item.title );
			treeitem->setToolTip( item.help );
			/*{
				QVariant v;
				v.setValue( item );
				qDebug() << v.type() << v.typeName();
				treeitem->setData( v, Qt::DisplayRole );
			}*/
			_items.append( treeitem );
			QStandardItem* catitem = 0;
			foreach( QStandardItem* tmp, _categories )
				if ( tmp->text() == cat )
					catitem = tmp;
			if ( catitem == 0 ) {
				QStandardItem* top = new QStandardItem( cat );
				top->setBackground( palette().alternateBase() );
				_model->appendRow( top );
				_categories.append( top );
				catitem = top;
			}
			catitem->appendRow( treeitem );
		}
	}
}

FancyListDialogDelegate::FancyListDialogDelegate( QObject* p ) : QItemDelegate( p ) {
	setClipping( true );
}
void FancyListDialogDelegate::paint( QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index ) const {
	QVariant data = index.data( Qt::DisplayRole );
	if ( data.type() == QVariant::UserType ) {
		drawBackground( painter, option, index );
		painter->save();

		//qDebug() << "FancyListDialogDelegate::paint(" << painter << "," << option << "," << index << ")";
		//qDebug() << " Should deal with this myself.";
		NSCore::GuiServer_Interface::ListItem item = data.value<NSCore::GuiServer_Interface::ListItem>();
		//painter->drawText( option.rect, QString( "<qt><b>%1</b> %2</qt>" ).arg( item.title ).arg( item.help ) );
		QTextDocument doc;
		if ( option.state & QStyle::State_Selected )
			doc.setHtml( QString( "<qt><b>%1</b> %2</qt>" ).arg( item.title ).arg( item.help ) );
		else
			doc.setHtml( item.title );
		//doc.setHtml( QString( "<qt><b>%1</b> %2</qt>" ).arg( item.title ).arg( item.help ) );
		doc.setTextWidth( option.rect.width() );
		QAbstractTextDocumentLayout* layout = doc.documentLayout();
		painter->translate( option.rect.x(), option.rect.y() );
		QAbstractTextDocumentLayout::PaintContext context;
		context.clip = option.rect.adjusted( -option.rect.x(), -option.rect.y(), 0,0 );
		layout->draw( painter, context );

		painter->restore();
	} else
		QItemDelegate::paint( painter, option, index );
}
QSize FancyListDialogDelegate::sizeHint( const QStyleOptionViewItem& option, const QModelIndex& index ) const {
	QVariant data = index.data( Qt::DisplayRole );
	if ( data.type() == QVariant::UserType ) {
		//qDebug() << "FancyListDialogDelegate::sizeHint(" << option << "," << index << ")";
		//qDebug() << "FancyListDialogDelegate::paint(" << painter << "," << option << "," << index << ")";
		//qDebug() << " Should deal with this myself.";
		NSCore::GuiServer_Interface::ListItem item = data.value<NSCore::GuiServer_Interface::ListItem>();
		//painter->drawText( option.rect, QString( "<qt><b>%1</b> %2</qt>" ).arg( item.title ).arg( item.help ) );
		QTextDocument doc;
		doc.setTextWidth( 400/*option.rect.width()*/ );
		if ( option.state & QStyle::State_Selected )
			doc.setHtml( QString( "<qt><b>%1</b> %2</qt>" ).arg( item.title ).arg( item.help ) );
		else
			doc.setHtml( item.title );
		//QAbstractTextDocumentLayout* layout = doc.documentLayout();
		qDebug() << " option.rect =" << option.rect;
		qDebug() << " doc.size =" << doc.size();
		return doc.size().toSize();
	} else
		return QItemDelegate::sizeHint( option, index );
}

FancyListView::FancyListView( QWidget* p ) : QAbstractItemView( p ) {
	qDebug() << "FancyListView::FancyListView(" << p << ")";
	viewport()->setLayout( new QVBoxLayout( viewport() ) );
	//setViewport( container = new SettingsContainer() );
	//viewport()->setMinimumSize( QSize( 200, 200 ) );
}
QRect FancyListView::visualRect( const QModelIndex& index ) const {
	qDebug() << "FancyListView::visualRect(" << index << ")";
	return QRect();
}
void FancyListView::scrollTo( const QModelIndex& index, QAbstractItemView::ScrollHint hint ) {
	qDebug() << "FancyListView::scrollTo(" << index << "," << hint << ")";
}
QModelIndex FancyListView::indexAt( const QPoint& point ) const {
	qDebug() << "FancyListView::indexAt(" << point << ")";
	return QModelIndex();
}
QModelIndex FancyListView::moveCursor( QAbstractItemView::CursorAction action, Qt::KeyboardModifiers modifiers ) {
	qDebug() << "FancyListView::moveCursor(" << action << "," << modifiers << ")";
	return QModelIndex();
}

void FancyListView::setModel( QAbstractItemModel* model ) {
	qDebug() << "FancyListView::setModel(" << model << ")";
	QAbstractItemView::setModel( model );

	qDebug() << "cols =" << model->columnCount() << " rows=" << model->rowCount();
#if 0
	for ( int j=0; j<model->columnCount(); ++j ) {
		for ( int i=0; i<model->rowCount(); ++i ) {
			QModelIndex index = model->index( i, j );
			qDebug() << " Will create label with text" << index.data().toString();
			container->insertWidget( new QLabel( index.data().toString() ), index.data().toString() );
			/*QLabel* tmp = new QLabel( index.data().toString(), viewport() );
			tmp->setLineWidth( 2 );
			viewport()->layout()->addWidget( tmp );
			tmp->show();*/
		}
	}
#endif
}
void FancyListView::dataChanged( const QModelIndex& topleft, const QModelIndex& bottomright ) {
	qDebug() << "FancyListView::dataChanged(" << topleft << "," << bottomright << ")";
}
void FancyListView::rowsInserted( const QModelIndex& parent, int begin, int end ) {
	qDebug() << "FancyListView::rowsInserted(" << parent << "," << begin << "," << end << ")";
	if ( ! parent.isValid() ) {
		SettingsContainer* tmp = new SettingsContainer( viewport() );
		viewport()->layout()->addWidget( tmp );
		tmp->show();
		containers.append( tmp );
	} else {
		int row = parent.row();
		SettingsContainer* tmp = containers.at( row );
		QModelIndex index = model()->index( row, 0, parent );
		CollapsibleWidget* w = new CollapsibleWidget( viewport() );
		w->setInnerWidget( new QLabel( QString( "<qt><b>%1</b></qt>" ).arg( index.data().toString() ), tmp ) );
		w->setExpanded( true );
		tmp->insertWidget( w, "bla" );
		tmp->show();
	}
}


