/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "qhexspinbox.h"
#include "qhexspinbox.moc"

#include <QtCore/QDebug>
#include <QtGui/QLineEdit>

QHexSpinBox::QHexSpinBox( QWidget* p )
	: QAbstractSpinBox( p )
	, _value( 0 )
	, _validator( new QRegExpValidator( QRegExp( "[0-9A-Fa-f]{1,2}" ), this ) )
{
	lineEdit()->setText( "0" );
	connect( this, SIGNAL( editingFinished() ), this, SLOT( finishedEditing() ) );
}

int QHexSpinBox::value() const {
	return _value;
}

void QHexSpinBox::stepBy( int step ) {
	finishedEditing();
	_value = qMax( qMin( _value+step, 255 ), 0 );
	lineEdit()->setText( QString::number( _value, 16 ) );
}

QValidator::State QHexSpinBox::validate( QString& string, int& pos ) const {
	return _validator->validate( string, pos );
}

void QHexSpinBox::finishedEditing() {
	_value = lineEdit()->text().toInt( 0, 16 );
}

QAbstractSpinBox::StepEnabled QHexSpinBox::stepEnabled() const {
	QAbstractSpinBox::StepEnabled ret = 0;
	if ( _value < 255 ) ret |= StepUpEnabled;
	if ( _value > 0 ) ret |= StepDownEnabled;
	return ret;
}

