/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef QHEXSPINBOX_H
#define QHEXSPINBOX_H

#include <QtGui/QAbstractSpinBox>

class QRegExpValidator;

/**
 * @brief This class provides a spinbox for one-byte hex-values.
 *
 * Use it the same as any other QAbstractSpinBox-derivate.
 */
class QHexSpinBox : public QAbstractSpinBox
{
	Q_OBJECT
	public:

		/**
		 * @brief Constructs a new QHexSpinBox
		 */
		QHexSpinBox( QWidget* );

		/**
		 * @brief The current value
		 */
		int value() const;

		/**
		 * @brief Change the value by step
		 */
		void stepBy( int );

		/**
		 * @brief Validates current input
		 *
		 * The given text is validated against a QRegExp of [ 0-9A-Fa-f ]{1,2}
		 * which gives one-byte hexadecimal numbers.
		 */
		QValidator::State validate( QString&, int& ) const;

	private slots:
		/**
		 * @brief Editing finished
		 *
		 * This sets the value to the one from the lineEdit.
		 * Is connected / called from QAbstractSpinBox::editingFinished()
		 */
		void finishedEditing();

	private:

		/**
		 * @brief If stepping is enabled
		 */
		StepEnabled stepEnabled() const;

		/**
		 * @brief stores the value internally
		 */
		int _value;

		/**
		 * @brief the validator for texts
		 */
		QRegExpValidator* _validator;
};

#endif // QHEXSPINBOX_H

