/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/
#ifndef LOWPASS_H
#define LOWPASS_H

#include <plugins_interface.h>

#include <vector>

class LowPass : public NSEvaluation::Modifier
{
	public:
		LowPass( NSCore::QTreeItem* );

		void modify( NSCore::AcqisitionDataSet&, int );

		QString type() const { return "LowPass"; }

		struct triple {
			double a, b, c;
			triple( double x, double y, double z ) : a( x ), b( y ), c( z ) {}
		};

		static std::vector<LowPass::triple> lpdes( double fc, double t, int ns );

	private:
		struct cache_t {
			int ns;
			int samples;
			double freq;

			std::vector<triple> coef;
			std::vector<double*> buffers;
			double v[ 3 ];

			cache_t() : ns( 0 ) {}

			void push_v( double n ) {
				v[ 0 ] = v[ 1 ];
				v[ 1 ] = v[ 2 ];
				v[ 2 ] = n;
			}
			double weighted_v() {
				return v[ 2 ] + 2 * v[ 1 ] + v[ 0 ];
			}
		};
		cache_t _cache;
};

#endif // LOWPASS_H
