/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef SCRIPTSNIPPET_H
#define SCRIPTSNIPPET_H

#include <QtGui/QWidget>

namespace NSScript {

	class ScriptSnippet_Private;

	/**
	 * @brief A snippet of QScript-code.
	 *
	 * @todo Implement undo/redo
	 */
	class ScriptSnippet : public QWidget
	{
		Q_OBJECT
		public:
			enum SaveLocation {
				SystemWide = 0,
				OwnJsFile = 1,
				PathXML = 2
			};

			/// constructor
			/**
			 * @param p Parent QWidget
			 */
			ScriptSnippet( QWidget* p );
			/// destructor
			~ScriptSnippet();

			/// Return the current code
			QString saveCode() const;
			/// Set the code
			void setCode( QString );

			/// Return the kind of snippet
			SaveLocation getSaveLocation() const { return _location; }
			/// Set the kind of snippet
			void setSaveLocation( SaveLocation );

			/// Return the name of this snippet
			QString name() const;
			/// Set the name of this snippet
			void setName( QString );

		public slots:
			/**
			 * @internal
			 * @{
			 */
			void on_btnExec_clicked();

			void on_btnUndo_clicked();
			void on_btnRedo_clicked();

			void on_btnSave_clicked();
			void on_comboBoxLocation_activated( int );
			/** @} */

		signals:
			/**
			 * @brief Emit the code that is to be executed
			 */
			void execScript( QString );

			/**
			 * @brief This Snippet is destroyed.
			 *
			 * Same as QObject::destroyed except for the arguments type. Because
			 * the QObjects signal is emitted in ~QObject there is no
			 * ScriptSnippet anymore and every upcast fails.
			 */
			void destroyed( ScriptSnippet* );

			/**
			 * @brief Request the system ( normally the ScriptEnvironment ) to save the code
			 */
			void systemSaveRequest();

		private:
			ScriptSnippet_Private* _p;
			SaveLocation _location;
	};

};

#endif // SCRIPTSNIPPET_H

